﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace IdentificadorDeTrucades
{
    class PopupTInt : System.Windows.Forms.Form
    {
        #region Atributs de la classe PopupTInt

        protected Popup p = new Popup();

        //Rectangles del contingut dels textes del popup
        protected Rectangle RectangleNom;
        protected Rectangle RectangleNum;
        protected Rectangle RectangleDept;
        protected Rectangle RectangleMail;

        //Events del Popup
        protected int nEventsMostrar;
        protected int nEventsAmagar;
        protected int nEventsVisible;
        protected int nIncrementMostrar;
        protected int nIncrementAmagar;

        //Events de ratoli
        protected bool bSiRatoliSobrePopup = false;
        protected bool bSiRatoliSobreTanca = false;
        protected bool bSiRatoliSobreNom = false;
        protected bool bSiRatoliSobreNum = false;
        protected bool bSiRatoliSobreMail = false;
        protected bool bSiRatoliSobreDept = false;
        protected bool bSiRatoliSobreHistorial = false;
        protected bool bSiRatoliApretat = false;
        protected bool bMantenirSiRatoli = true;
        protected bool bTornarAMostrarSiRatoli = true;

        //Timer necessari per fer les animacions del PopupTInt
        protected Timer timer = new Timer();

        //Variable per definir els diferents estats que pot presentar el PopupTInt
        protected EstatsPopupTInt estatPopupTInt = EstatsPopupTInt.amagat;

        //Rectangles on aniran representats els textes del PopupTInt
        public Rectangle RectNom;
        public Rectangle RectNum;
        public Rectangle RectDept;
        public Rectangle RectMail;
        protected Rectangle RectangleAreaDeTreball;

        //Definico de les parts clicables del PopupTInt
        public bool NomClicable = false;
        public bool NumClicable = true;
        public bool DeptClicable = true;
        public bool MailClicable = true;
        public bool TancaClicable = true;
        public bool HistorialClicable = true;

        public bool ActivarSeleccioRectangle = true;

        //Events del PopupTInt
        public event EventHandler NomClic = null;
        public event EventHandler NumClic = null;
        public event EventHandler DeptClic = null;
        public event EventHandler MailClic = null;
        public event EventHandler TancaClic = null;
        public event EventHandler HistorialClic = null;

        #endregion

        #region Estats del PopupTInt

        public enum EstatsPopupTInt
        {
            amagat = 0,
            apareguent = 1,
            visible = 2,
            desapareguent = 3
        }

        #endregion 

        #region Constructor de la classe PopupTInt

        public PopupTInt()
        {
            FormBorderStyle = FormBorderStyle.None;
            WindowState = FormWindowState.Minimized;
            base.Show();
            base.Hide();
            WindowState = FormWindowState.Normal;
            ShowInTaskbar = false;
            TopMost = true;
            MaximizeBox = false;
            MinimizeBox = false;
            ControlBox = false;

            timer.Enabled = true;
            timer.Tick += new EventHandler(OnTimer);
        }

        #endregion

        #region Funcions de la classe PopupTInt

        //Retorna l'estat del PopupTInt
        public EstatsPopupTInt EstatPopupTInt
        {
            get { return estatPopupTInt; }
            set { estatPopupTInt = value; }
        }

        //Indica si el PupUp ha de romandre visible quan te el ratoli a sobre
        public bool MantenirSiRatoli
        {
            get{return bMantenirSiRatoli;}
            set{bMantenirSiRatoli = value;}
        }

        //Indica si el PopUp ha de re-apareixer si passem el ratoli per sobre quan esta desapareguent
        public bool TornarAMostrarSiRatoli
        {
            get
            {return bTornarAMostrarSiRatoli;}
            set{bTornarAMostrarSiRatoli = value;}
        }

        [DllImport("user32.dll")]
        private static extern Boolean ShowWindow(IntPtr hWnd, Int32 nCmdShow);

        //Funcio per mostrar el PopUp
        public void Show(string strNom, string strNum, string strDept, string strMail, int nTempsEnApareixer, int nTempsMantenir, int nTempsEnAmagar)
        {
            RectangleAreaDeTreball = Screen.GetWorkingArea(RectangleAreaDeTreball);

            p.ObtenirTextNom = strNom;
            p.ObtenirTextNum = strNum;
            p.ObtenirTextDept = strDept;
            p.ObtenirTextMail = strMail;

            nEventsVisible = nTempsMantenir;

            CalcularRectanglesRatoli();

            //Calculo l'increment de pixels i el valor del timer per fer l'animacio que motra el PopUp
            int nEvents;
            if (nTempsEnApareixer > 10)
            {
                nEvents = Math.Min((nTempsEnApareixer / 10), p.ObtenirImFons.Height);
                nEventsMostrar = nTempsEnApareixer / nEvents;
                nIncrementMostrar = p.ObtenirImFons.Height / nEvents;
            }
            else
            {
                nEventsMostrar = 10;
                nIncrementMostrar = p.ObtenirImFons.Height;
            }

            //Faig el mateix per fer l'animacio que amaga el PopUp
            if (nTempsEnAmagar > 10)
            {
                nEvents = Math.Min((nTempsEnAmagar / 10), p.ObtenirImFons.Height);
                nEventsAmagar = nTempsEnAmagar / nEvents;
                nIncrementAmagar = p.ObtenirImFons.Height / nEvents;
            }
            else
            {
                nEventsAmagar = 10;
                nIncrementAmagar = p.ObtenirImFons.Height;
            }

            switch (estatPopupTInt)
            {
                case EstatsPopupTInt.amagat:
                    estatPopupTInt = EstatsPopupTInt.apareguent;
                    SetBounds(RectangleAreaDeTreball.Right - p.ObtenirImFons.Width - 17, RectangleAreaDeTreball.Bottom - 1, p.ObtenirImFons.Width, 0);
                    timer.Interval = nEventsMostrar;
                    timer.Start();
                    
                    ShowWindow(this.Handle, 4);
                    break;

                case EstatsPopupTInt.apareguent:
                    Refresh();
                    break;

                case EstatsPopupTInt.visible:
                    timer.Stop();
                    timer.Interval = nEventsVisible;
                    timer.Start();
                    Refresh();
                    break;

                case EstatsPopupTInt.desapareguent:
                    timer.Stop();
                    estatPopupTInt = EstatsPopupTInt.visible;
                    SetBounds(RectangleAreaDeTreball.Right - p.ObtenirImFons.Width - 17, RectangleAreaDeTreball.Bottom - p.ObtenirImFons.Height - 1, p.ObtenirImFons.Width, p.ObtenirImFons.Height);
                    timer.Interval = nEventsVisible;
                    timer.Start();
                    Refresh();
                    break;
            }
        }

        //Funcio per amagar el PopUp
        public new void Hide()
        {
            if (estatPopupTInt != EstatsPopupTInt.amagat)
            {
                timer.Stop();
                estatPopupTInt = EstatsPopupTInt.amagat;
                base.Hide();
            }
        }

        //Estableix la imatge de fons del PopupTInt a partir d'un nom de fitxer
        public void SetImFonsPopupTInt(string strNomFitxer, Color color)
        {
            p.ObtenirImFons = new Bitmap(strNomFitxer);
            Width = p.ObtenirImFons.Width;
            Height = p.ObtenirImFons.Height;
            Region = ImatgeARegio(p.ObtenirImFons, color);

        }

        //Estableix la imatge de fons del PopUp a partir d'una imatge
        public void SetImFonsPopupTInt(Image imatge, Color color)
        {
            p.ObtenirImFons = new Bitmap(imatge);
            Width = p.ObtenirImFons.Width;
            Height = p.ObtenirImFons.Height;
            Region = ImatgeARegio(p.ObtenirImFons, color);
        }

        //Estableix la imatge del boto de tancar a partir d'un nom de fitxer
        public void SetImBotoTanca(string strNomFitxer, Color color, Point posicio)
        {
            p.ObtenirImBotoTancar = new Bitmap(strNomFitxer);
            p.ObtenirImBotoTancar.MakeTransparent(color);
            p.ObtenirTamBotoTancar = new Size(p.ObtenirImBotoTancar.Width / 3, p.ObtenirImBotoTancar.Height);
            p.ObtenirLocBotoTancar = posicio;
        }

        //Estableix la imatge del boto de tancar a partir d'una imatge
        public void SetImBotoTanca(Image imagte, Color color, Point posicio)
        {
            p.ObtenirImBotoTancar = new Bitmap(imagte);
            p.ObtenirImBotoTancar.MakeTransparent(color);
            p.ObtenirTamBotoTancar = new Size(p.ObtenirImBotoTancar.Width / 3, p.ObtenirImBotoTancar.Height);
            p.ObtenirLocBotoTancar = posicio;
        }

        //Estableix la imatge del boto historial a partir d'un nom de fitxer
        public void SetImBotoHistorial(string strNomFitxer, Color color, Point posicio)
        {
            p.ObtenirImBotoHistorial = new Bitmap(strNomFitxer);
            p.ObtenirImBotoHistorial.MakeTransparent(color);
            p.ObtenirTamBotoHistorial = new Size(p.ObtenirImBotoHistorial.Width / 3, p.ObtenirImBotoHistorial.Height);
            p.ObtenirLocBotoHistorial = posicio;
        }

        //Estableix la imatge del boto historial a partir d'una imatge
        public void SetImBotoHistorial(Image imagte, Color color, Point posicio)
        {
            p.ObtenirImBotoHistorial = new Bitmap(imagte);
            p.ObtenirImBotoHistorial.MakeTransparent(color);
            p.ObtenirTamBotoHistorial = new Size(p.ObtenirImBotoHistorial.Width / 3, p.ObtenirImBotoHistorial.Height);
            p.ObtenirLocBotoHistorial = posicio;
        }

        //Estableix la foto de la persona que truca a partir d'un nom de fitxer
        public void SetImFoto(string strNomFitxer, Color color, Point posicio)
        {
            p.ObtenirImFoto = new Bitmap(strNomFitxer);
            p.ObtenirImFoto.MakeTransparent(color);
            p.ObtenirTamFoto = new Size(p.ObtenirImFoto.Width, p.ObtenirImFoto.Height);
            p.ObtenirLocFoto = posicio;
        }

        //Estableix la foto de la persona que truca a partir d'una imatge
        public void SetImFoto(Image imatge, Color color, Point posicio)
        {
            p.ObtenirImFoto = new Bitmap(imatge);
            p.ObtenirImFoto.MakeTransparent(color);
            p.ObtenirTamFoto = new Size(p.ObtenirImFoto.Width, p.ObtenirImFoto.Height);
            p.ObtenirLocFoto = posicio;
        }

        //Dibuixa el boto de tancar
        protected void DibuixaBotoTancar(Graphics grfx)
        {
            if (p.ObtenirImBotoTancar != null)
            {
                Rectangle rectDest = new Rectangle(p.ObtenirLocBotoTancar, p.ObtenirTamBotoTancar);
                Rectangle rectSrc;

                if (bSiRatoliSobreTanca)
                {
                    if (bSiRatoliApretat)
                        rectSrc = new Rectangle(new Point(p.ObtenirTamBotoTancar.Width * 2, 0), p.ObtenirTamBotoTancar);
                    else
                        rectSrc = new Rectangle(new Point(p.ObtenirTamBotoTancar.Width, 0), p.ObtenirTamBotoTancar);
                }
                else
                    rectSrc = new Rectangle(new Point(0, 0), p.ObtenirTamBotoTancar);


                grfx.DrawImage(p.ObtenirImBotoTancar, rectDest, rectSrc, GraphicsUnit.Pixel);
            }
        }

        //Dibuixa el boto de l'historial
        protected void DibuixaBotoHistorial(Graphics grfx)
        {
            if (p.ObtenirImBotoHistorial != null)
            {
                Rectangle rectDest = new Rectangle(p.ObtenirLocBotoHistorial, p.ObtenirTamBotoHistorial);
                Rectangle rectSrc;

                if (bSiRatoliSobreHistorial)
                {
                    if (bSiRatoliApretat)
                        rectSrc = new Rectangle(new Point(p.ObtenirTamBotoHistorial.Width * 2, 0), p.ObtenirTamBotoHistorial);
                    else
                        rectSrc = new Rectangle(new Point(p.ObtenirTamBotoHistorial.Width, 0), p.ObtenirTamBotoHistorial);
                }
                else
                    rectSrc = new Rectangle(new Point(0, 0), p.ObtenirTamBotoHistorial);


                grfx.DrawImage(p.ObtenirImBotoHistorial, rectDest, rectSrc, GraphicsUnit.Pixel);
            }
        }

        //Dibuixa la foto de la persona que truca
        protected void DibuixaFoto(Graphics grfx)
        {
            if (p.ObtenirImFoto != null)
            {

                Rectangle rectDest = new Rectangle(new Point(p.ObtenirTamFoto.Width * 2, 0), p.ObtenirTamFoto);
                Rectangle rectSrc;

                rectSrc = new Rectangle(new Point(0, 0), p.ObtenirTamFoto);

                grfx.DrawImage(p.ObtenirImFoto, rectDest, rectSrc, GraphicsUnit.Pixel);
            }
        }

        //Dibuixa un text
        protected void DibuixaText(Graphics grfx)
        {
            //Si es el text del nom de la persona que truca
            if (p.ObtenirTextNom != null && p.ObtenirTextNom.Length != 0)
            {
                StringFormat sf = new StringFormat();
                sf.Alignment = StringAlignment.Near;
                sf.LineAlignment = StringAlignment.Center;
                sf.FormatFlags = StringFormatFlags.NoWrap;
                sf.Trimming = StringTrimming.EllipsisCharacter;

                if (bSiRatoliSobreNom)
                    grfx.DrawString(p.ObtenirTextNom, p.ObtenirFontEventNom, new SolidBrush(p.ObtenirColEventNom), RectNom, sf);
                else
                    grfx.DrawString(p.ObtenirTextNom, p.ObtenirFontNormalNom, new SolidBrush(p.ObtenirColNormalNom), RectNom, sf);
            }
            
            //Si es el text del num de la persona que truca
            if (p.ObtenirTextNum != null && p.ObtenirTextNum.Length != 0)
            {
                StringFormat sf = new StringFormat();
                sf.Alignment = StringAlignment.Near;
                sf.LineAlignment = StringAlignment.Center;
                sf.FormatFlags = StringFormatFlags.NoWrap;
                sf.Trimming = StringTrimming.EllipsisCharacter;

                if (bSiRatoliSobreNum)
                    grfx.DrawString(p.ObtenirTextNum, p.ObtenirFontEventNum, new SolidBrush(p.ObtenirColEventNum), RectNum, sf);
                else
                    grfx.DrawString(p.ObtenirTextNum, p.ObtenirFontNormalNum, new SolidBrush(p.ObtenirColNormalNum), RectNum, sf);
            }

            //Si es el text del departament de la persona que truca
            if (p.ObtenirTextDept != null && p.ObtenirTextDept.Length != 0)
            {
                StringFormat sf = new StringFormat();
                sf.Alignment = StringAlignment.Near;
                sf.LineAlignment = StringAlignment.Center;
                sf.FormatFlags = StringFormatFlags.NoWrap;
                sf.Trimming = StringTrimming.EllipsisCharacter;

                if (bSiRatoliSobreDept)
                    grfx.DrawString(p.ObtenirTextDept, p.ObtenirFontEventDept, new SolidBrush(p.ObtenirColEventDept), RectDept, sf);
                else
                    grfx.DrawString(p.ObtenirTextDept, p.ObtenirFontNormalDept, new SolidBrush(p.ObtenirColNormalDept), RectDept, sf);
            }

            //Si es el text del correu electronic de la persona que truca
            if (p.ObtenirTextMail != null && p.ObtenirTextMail.Length != 0)
            {
                StringFormat sf = new StringFormat();
                sf.Alignment = StringAlignment.Near;
                sf.LineAlignment = StringAlignment.Center;
                sf.FormatFlags = StringFormatFlags.NoWrap;
                sf.Trimming = StringTrimming.EllipsisCharacter;

                if (bSiRatoliSobreMail)
                    grfx.DrawString(p.ObtenirTextMail, p.ObtenirFontEventMail, new SolidBrush(p.ObtenirColEventMail), RectMail, sf);
                else
                    grfx.DrawString(p.ObtenirTextMail, p.ObtenirFontNormalMail, new SolidBrush(p.ObtenirColNormalDept), RectMail, sf);
            }
        }

        //Funcio per ajustar els tamanys dels rectangles al text
        protected void CalcularRectanglesRatoli()
        {
            Graphics grfx = CreateGraphics();
            StringFormat sf = new StringFormat();
            sf.Alignment = StringAlignment.Center;
            sf.LineAlignment = StringAlignment.Center;
            sf.FormatFlags = StringFormatFlags.MeasureTrailingSpaces;

            SizeF sizefNom = grfx.MeasureString(p.ObtenirTextNom, p.ObtenirFontEventNom, RectNom.Width, sf);
            SizeF sizefNum = grfx.MeasureString(p.ObtenirTextNum, p.ObtenirFontEventNum, RectNum.Width, sf);
            SizeF sizefDept = grfx.MeasureString(p.ObtenirTextDept, p.ObtenirFontEventDept, RectDept.Width, sf);
            SizeF sizefMail = grfx.MeasureString(p.ObtenirTextMail, p.ObtenirFontEventMail, RectMail.Width, sf);

            grfx.Dispose();

            //Mro si el text cap a dintre del rectangle
            if (sizefNom.Height > RectangleNom.Height)
            {
                RectangleNom = new Rectangle(RectNom.Left, RectNom.Top, RectNom.Width, RectNom.Height);
            }
            else
            {
                RectangleNom = new Rectangle(RectNom.Left, RectNom.Top, (int)sizefNom.Width, (int)sizefNom.Height);
            }
            RectangleNom.Inflate(0, 2);

            if (sizefNum.Height > RectangleNum.Height)
            {
                RectangleNum = new Rectangle(RectNum.Left, RectNum.Top, RectNum.Width, RectNum.Height);
            }
            else
            {
                RectangleNum = new Rectangle(RectNum.Left, RectNum.Top, (int)sizefNum.Width, (int)sizefNum.Height);
            }
            RectangleNum.Inflate(0, 2);

            if (sizefDept.Height > RectangleDept.Height)
            {
                RectangleDept = new Rectangle(RectDept.Left, RectDept.Top, RectDept.Width, RectDept.Height);
            }
            else
            {
                RectangleDept = new Rectangle(RectDept.Left, RectDept.Top, (int)sizefDept.Width, (int)sizefDept.Height);
            }
            RectangleDept.Inflate(0, 2);

            if (sizefMail.Height > RectangleMail.Height)
            {
                RectangleMail = new Rectangle(RectMail.Left, RectMail.Top, RectMail.Width, RectMail.Height);
            }
            else
            {
                RectangleMail = new Rectangle(RectMail.Left, RectMail.Top, (int)sizefMail.Width, (int)sizefMail.Height);
            }
            RectangleMail.Inflate(0, 2);
        }

        //Funcio per posar una imatge a una regio
        protected Region ImatgeARegio(Bitmap imatge, Color color)
        {
            if (imatge == null)
                throw new ArgumentNullException("Bitmap", "Bitmap cannot be null!");

            int height = imatge.Height;
            int width = imatge.Width;

            GraphicsPath path = new GraphicsPath();

            for (int j = 0; j < height; j++)
                for (int i = 0; i < width; i++)
                {
                    if (imatge.GetPixel(i, j) == color)
                        continue;

                    int x0 = i;

                    while ((i < width) && (imatge.GetPixel(i, j) != color))
                        i++;

                    path.AddRectangle(new Rectangle(x0, j, i - x0, 1));
                }

            Region region = new Region(path);
            path.Dispose();
            return region;
        }

        //Funcio del Timer
        protected void OnTimer(Object obj, EventArgs ea)
        {
            switch (estatPopupTInt)
            {
                case EstatsPopupTInt.apareguent:
                    if (Height < p.ObtenirImFons.Height)
                        SetBounds(Left, Top - nIncrementMostrar, Width, Height + nIncrementMostrar);
                    else
                    {
                        timer.Stop();
                        Height = p.ObtenirImFons.Height;
                        timer.Interval = nEventsVisible;
                        estatPopupTInt = EstatsPopupTInt.visible;
                        timer.Start();
                    }
                    break;

                case EstatsPopupTInt.visible:
                    timer.Stop();
                    timer.Interval = nEventsAmagar;
                    
                    if ((bMantenirSiRatoli && !bSiRatoliSobrePopup) || (!bMantenirSiRatoli))
                    {
                        estatPopupTInt = EstatsPopupTInt.desapareguent;
                    }
                    
                    timer.Start();
                    break;

                case EstatsPopupTInt.desapareguent:
                    
                    if (bTornarAMostrarSiRatoli && bSiRatoliSobrePopup)
                    {
                        estatPopupTInt = EstatsPopupTInt.apareguent;
                    }
                    else
                    {
                        if (Top < RectangleAreaDeTreball.Bottom)
                            SetBounds(Left, Top + nIncrementAmagar, Width, Height - nIncrementAmagar);
                        else
                            Hide();
                    }
                    break;
            }

        }

        //Funcions dels diferents ecents de ratolí
        protected override void OnMouseEnter(EventArgs ea)
        {
            base.OnMouseEnter(ea);
            bSiRatoliSobrePopup = true;
            Refresh();
        }

        protected override void OnMouseLeave(EventArgs ea)
        {
            base.OnMouseLeave(ea);
            bSiRatoliSobrePopup = false;
            bSiRatoliSobreTanca = false;
            bSiRatoliSobreNom = false;
            bSiRatoliSobreNum = false;
            bSiRatoliSobreDept = false;
            bSiRatoliSobreMail = false;
            bSiRatoliSobreHistorial = false;

            Refresh();
        }

        protected override void OnMouseMove(MouseEventArgs mea)
        {
            base.OnMouseMove(mea);

            bool bContentModified = false;

            if ((mea.X > p.ObtenirLocBotoTancar.X) && (mea.X < p.ObtenirLocBotoTancar.X + p.ObtenirTamBotoTancar.Width) && (mea.Y > p.ObtenirLocBotoTancar.Y) && (mea.Y < p.ObtenirLocBotoTancar.Y + p.ObtenirTamBotoTancar.Height) && TancaClicable)
            {
                if (!bSiRatoliSobreTanca)
                {
                    bSiRatoliSobreTanca = true;
                    bSiRatoliSobreNom = false;
                    bSiRatoliSobreNum = false;
                    bSiRatoliSobreDept = false;
                    bSiRatoliSobreMail = false;
                    bSiRatoliSobreHistorial = false;
                    Cursor = Cursors.Hand;
                    bContentModified = true;
                }
            }
            else if ((mea.X > p.ObtenirLocBotoHistorial.X) && (mea.X < p.ObtenirLocBotoHistorial.X + p.ObtenirTamBotoHistorial.Width) && (mea.Y > p.ObtenirLocBotoHistorial.Y) && (mea.Y < p.ObtenirLocBotoHistorial.Y + p.ObtenirTamBotoHistorial.Height) && HistorialClicable)
            {
                if (!bSiRatoliSobreHistorial)
                {
                    bSiRatoliSobreTanca = false;
                    bSiRatoliSobreNom = false;
                    bSiRatoliSobreNum = false;
                    bSiRatoliSobreDept = false;
                    bSiRatoliSobreMail = false;
                    bSiRatoliSobreHistorial = true;
                    Cursor = Cursors.Hand;
                    bContentModified = true;
                }
            }
            else if (RectangleNom.Contains(new Point(mea.X, mea.Y)) && NomClicable)
            {
                if (!bSiRatoliSobreNom)
                {
                    bSiRatoliSobreTanca = false;
                    bSiRatoliSobreNom = true;
                    bSiRatoliSobreNum = false;
                    bSiRatoliSobreDept = false;
                    bSiRatoliSobreMail = false;
                    bSiRatoliSobreHistorial = false;
                    Cursor = Cursors.Hand;
                    bContentModified = true;
                }
            }
            else if (RectangleNum.Contains(new Point(mea.X, mea.Y)) && NumClicable)
            {
                if (!bSiRatoliSobreNum)
                {
                    bSiRatoliSobreTanca = false;
                    bSiRatoliSobreNom = false;
                    bSiRatoliSobreNum = true;
                    bSiRatoliSobreDept = false;
                    bSiRatoliSobreMail = false;
                    bSiRatoliSobreHistorial = false;
                    Cursor = Cursors.Hand;
                    bContentModified = true;
                }
            }
            else if (RectangleDept.Contains(new Point(mea.X, mea.Y)) && DeptClicable)
            {
                if (!bSiRatoliSobreDept)
                {
                    bSiRatoliSobreTanca = false;
                    bSiRatoliSobreNom = false;
                    bSiRatoliSobreNum = false;
                    bSiRatoliSobreDept = true;
                    bSiRatoliSobreMail = false;
                    bSiRatoliSobreHistorial = false;
                    Cursor = Cursors.Hand;
                    bContentModified = true;
                }
            }
            else if (RectangleMail.Contains(new Point(mea.X, mea.Y)) && MailClicable)
            {
                if (!bSiRatoliSobreMail)
                {
                    bSiRatoliSobreTanca = false;
                    bSiRatoliSobreNom = false;
                    bSiRatoliSobreNum = false;
                    bSiRatoliSobreDept = false;
                    bSiRatoliSobreMail = true;
                    bSiRatoliSobreHistorial = false;
                    Cursor = Cursors.Hand;
                    bContentModified = true;
                }
            }
            else
            {
                if (bSiRatoliSobreTanca || bSiRatoliSobreNom || bSiRatoliSobreNum || bSiRatoliSobreDept || bSiRatoliSobreMail || bSiRatoliSobreHistorial)
                    bContentModified = true;

                bSiRatoliSobreTanca = false;
                bSiRatoliSobreNom = false;
                bSiRatoliSobreNum = false;
                bSiRatoliSobreDept = false;
                bSiRatoliSobreMail = false;
                bSiRatoliSobreHistorial = false;
                Cursor = Cursors.Default;
            }

            if (bContentModified)
                Refresh();
        }

        protected override void OnMouseDown(MouseEventArgs mea)
        {
            base.OnMouseDown(mea);
            bSiRatoliApretat = true;

            if (bSiRatoliSobreTanca)
                Refresh();
            if (bSiRatoliSobreHistorial)
                Refresh();
        }

        protected override void OnMouseUp(MouseEventArgs mea)
        {
            base.OnMouseUp(mea);
            bSiRatoliApretat = false;

            if (bSiRatoliSobreTanca)
            {
                Hide();

                if (TancaClic != null)
                    TancaClic(this, new EventArgs());
            }
            //Si es fa clic a l'historial
            else if (bSiRatoliSobreHistorial)
            {

                if (HistorialClic != null)
                    HistorialClic(this, new EventArgs());

            }
            //Si es fa clic al Nom
            else if (bSiRatoliSobreNom)
            {
                //En principi no es fa res

                if (NomClic != null)
                    NomClic(this, new EventArgs());
            }
            //Si es fa clic al Num
            else if (bSiRatoliSobreNum)
            {
                p.ObtenirTPNumPT.ClicTelf();

                if (NumClic != null)
                    NumClic(this, new EventArgs());
            }
            //Si es fa clic al departament
            else if (bSiRatoliSobreDept)
            {
                p.ObtenirTPDeptPT.ClicDept();

                if (DeptClic != null)
                    DeptClic(this, new EventArgs());
            }
            //Si es fa clic al correu electronic
            else if (bSiRatoliSobreMail)
            {
                p.ObtenirTPMailPT.ClicMail();

                if (MailClic != null)
                    MailClic(this, new EventArgs());
            }
        }

        protected override void OnPaintBackground(PaintEventArgs pea)
        {
            Graphics grfx = pea.Graphics;
            grfx.PageUnit = GraphicsUnit.Pixel;

            Graphics offScreenGraphics;
            Bitmap offscreenBitmap;

            offscreenBitmap = new Bitmap(p.ObtenirImFons.Width, p.ObtenirImFons.Height);
            offScreenGraphics = Graphics.FromImage(offscreenBitmap);

            if (p.ObtenirImFons != null)
            {
                offScreenGraphics.DrawImage(p.ObtenirImFons, 0, 0, p.ObtenirImFons.Width, p.ObtenirImFons.Height);
            }

            DibuixaBotoTancar(offScreenGraphics);
            //DibuixaFoto(offScreenGraphics);
            DibuixaBotoHistorial(offScreenGraphics);
            DibuixaText(offScreenGraphics);

            grfx.DrawImage(offscreenBitmap, 0, 0);
        }

        #endregion
    }
}
